/* Time Limit Milter by Tomas Rudh, tomas@rudh.mine.nu */
/* Add an existing address with a plus and a last use date in the format yymmdd */
/* Example: tomas+080124@rudh.mine.nu */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>
#include <time.h>

#include "libmilter/mfapi.h"

#ifndef true
typedef int bool;
# define false	0
# define true	1
#endif /* ! true */

char *tlversion = "1.0";
bool debug = false;

struct mlfiPriv
{
	bool bCopyMessage;
};

#define MLFIPRIV	((struct mlfiPriv *) smfi_getpriv(ctx))

extern sfsistat	 mlfi_cleanup(SMFICTX *);

sfsistat
mlfi_envrcpt(ctx, envfrom)
  SMFICTX *ctx;
  char **envfrom;
{
  char newaddr[100];		/* Address after the date is stripped */
  char *addrdate;		/* The date in the TO address */
  int iaddrdate;		/* The date in the TO address, in integer format (80124) */
  int itodaydate;		/* Todays date, in integer format (80103) */
  int i;			/* Loop variables */
  bool bailout = false;		/* True == not a time limited rcpt address */

  struct mlfiPriv *priv;
  char *rcpt_addr;
  bool accept = true;

  if (debug) fprintf(stderr, "mlfi_envrcpt\n");
  /* allocate some private memory */
  priv = malloc(sizeof *priv);
  if (priv == NULL)
  {
    /* can't accept this message right now */
    return SMFIS_TEMPFAIL;
  }

  memset(priv, '\0', sizeof *priv);
  priv->bCopyMessage = false;

  rcpt_addr = smfi_getsymval(ctx, "{rcpt_addr}");
  if (debug) fprintf(stderr, "  To address: %s\n", rcpt_addr);
  
  /* Check if TO address is long enough*/
  if( strlen( rcpt_addr ) > 7 )
  {
    /* Find the last '+' and get the text after it */
    addrdate = strrchr( rcpt_addr, '+' );
    if (debug) printf( "<%s>\n", addrdate );

    /* Only run if '+' is found */
    if (addrdate != NULL && strlen(addrdate) == 7)
    {
      /* Check that all six are numbers */
      for( i = 1; i < 7; i++ )
      {
        if( debug ) printf( "%d\n",addrdate[i] );
        if( isdigit( addrdate[i] ) == 0 )	/* 0 = not a number */
        {
          i = 100;			/* Exit loop */
	  bailout = true;
          if( debug ) printf( "Don't reject address!\n" );
        }
      }
      if (bailout == false)
      {
        /* Convert date to integer */
        iaddrdate = atoi( addrdate + 1 );
        if( debug ) printf( "Expire date: %i\n", iaddrdate );

        /* Get todays date */
        time_t tt = time(0);
        struct tm ttm = *localtime(&tt);
        itodaydate = (ttm.tm_year - 100) * 10000 + (ttm.tm_mon + 1) * 100 + ttm.tm_mday;
        if( debug ) printf( "Today: %d\n", itodaydate );

        /* Check if TO address has expired */
        if( iaddrdate < itodaydate ) accept = false;
      }
    }
  }
  if( debug ) printf( "Reject address: %i\n", accept );

  /* save the private data */
  smfi_setpriv(ctx, priv);

  /* continue processing */
  if (accept)
    return SMFIS_CONTINUE;
  else
  {
    mlfi_cleanup(ctx);
    return SMFIS_REJECT;
  }
}

sfsistat
mlfi_eom(ctx)
  SMFICTX *ctx;
{
  struct mlfiPriv *priv = MLFIPRIV;
	
  if (debug) fprintf(stderr, "mlfi_eom\n");
  if (smfi_addheader (ctx, "X-Time-Limit-Milter", tlversion) == MI_FAILURE)
    if (debug) printf("  Could not add header\n");
  return mlfi_cleanup(ctx);
}

sfsistat
mlfi_abort(ctx)
	SMFICTX *ctx;
{
	if (debug) fprintf(stderr, "mlfi_abort\n");
	return mlfi_cleanup(ctx);
}

sfsistat
mlfi_cleanup(ctx)
	SMFICTX *ctx;
{
	if (debug) printf("mlfi_cleanup\n");
	struct mlfiPriv *priv = MLFIPRIV;

	/* release private memory */
	free(priv);
	smfi_setpriv(ctx, NULL);
}

struct smfiDesc smfilter =
{
	"tlmilter",	/* filter name */
	SMFI_VERSION,	/* version code -- do not change */
	SMFIF_ADDRCPT|SMFIF_ADDHDRS|SMFIF_DELRCPT,	/* flags */
	NULL,		/* connection info filter */
	NULL,		/* SMTP HELO command filter */
	NULL,		/* envelope sender filter */
	mlfi_envrcpt,	/* envelope recipient filter */
	NULL,		/* header filter */
	NULL,		/* end of header */
	NULL,		/* body block filter */
	mlfi_eom,	/* end of message */
	mlfi_abort,	/* message aborted */
	NULL		/* connection cleanup */
};

int
main(argc, argv)
  int argc;
  char *argv[];
{
  int c;
  const char *args = "p:";

  /* Process command line options */
  while ((c = getopt(argc, argv, args)) != -1)
  {
    switch (c)
    {
    case 'p':
      if (optarg == NULL || *optarg == '\0')
      {
        (void) fprintf(stderr, "Illegal conn: %s\n", optarg);
        exit(EX_USAGE);
      }
      (void) smfi_setconn(optarg);
      break;
    }
  }
  remove (optarg);
  if (smfi_register(smfilter) == MI_FAILURE)
  {
    fprintf(stderr, "smfi_register failed\n");
    exit(EX_UNAVAILABLE);
  }
  return smfi_main();
}
/* eof */
